<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>All Posts - {{SITE_TITLE}}</title>
</head>
<body>
  <h1>All Posts<h1>  
      <ul>
        {% for page in pages %}
        <li><a href="/{{page.slug}}">{{page.title}}</a></li>
        {% endfor %}
      </ul>
</body>
</html>
