<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <title>{{SITE_TITLE}}</title>
</head>
<body>
  <div>
    <h1>Start your Engines!</h1>
    <img src="https://media0.giphy.com/media/12MOtvrbwX3DLG/giphy.gif" alt="engine-blows-man-over">
  </div>
  <div>
    <h2>Congratulations! You've Rendered your First Page</h2>
    <p>You may want to do a little more, but hopefully this will be a great starting point.</p>
    <p>If you have any questions on how Render Engine works, check out our <a
    href="render-engine.readthedocs.io">Documentation Page</a></p>
  </div>
</body>
</html>
